#
# Copyright 2015 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

class Api::BaseController < ActionController::Base
  include ApiWebCommon
  include CtfApiAuthentication
  include SharedTeamforgeApplication

  before_action :cors_set_access_control_headers

  rescue_from CanCan::AccessDenied, with: :unauthorized
  rescue_from SystemCallError, with: :service_unavailable

  # getting a project object and raise an exception with proper error message
  def get_project
    raise Exception.new("No project found") unless current_project
  rescue Exception => ex
    logger.error(ex.message)
    render_result({:status => 404, :message => ex.message}, 404)
  end

  # Renders response for any method.
  def render_result(result, status = 200)
    respond_to do |format|
      format.json { render :json => result, :status => status }
    end
  end

  # Return the CORS AC Headers for all responses
  def cors_set_access_control_headers
    headers['Access-Control-Allow-Origin'] = "*"
    headers['Access-Control-Allow-Methods'] = 'POST, GET, PUT, PATCH, DELETE, OPTIONS, HEAD'
    headers['Access-Control-Allow-Headers'] = 'X-Orchestrate-Session, X-CSRF-Token, Content-Type'
    headers['Access-Control-Max-Age'] = '1728000'
    head(:ok) if request.request_method == "OPTIONS"
  end


end
