#
# Copyright 2015 CollabNet, Inc. All rights reserved. Usage governed by the LICENSE.txt file.
#

# Controller for viewing and creation of Schemas via HTTP API
class Api::V1::SourcesController < Api::BaseController
  include RabbitServices
  before_action :authenticate_with_teamforge_and_get_permissions
  before_action :get_project
  before_action :require_crud_permissions

  def index
    sources = ActivitySource.by_project(current_project.id)
    sources = sources.by_tool(params[:tool_id]) if params[:tool_id]
    render json: { sources: sources.all }, status: :ok
  end

  def new
    render json: { source: ActivitySource.generated_columns }, status: :ok
  end

  def show
    source = ActivitySource.where(id: params[:id]).by_project(current_project.id).first
    if source
      render json: { source: source }, status: :ok
    else
      head :not_found
    end
  end

  def create
    source = ApiSourceAdapter.new(params[:source], current_project)
    if source.save
      create_queue_account(source.native_source.queue_username, source.native_source.queue_password)
      render json: { source: source }, status: :ok
    else
      render json: { errors: source }, status: :unprocessable_entity
    end
  end

  def update
    source_updater(params[:source])
  end

  def activate
    source_updater({active: true})
  end

  def deactivate
    source_updater({active: false})
  end

  def source_updater(values={})
    source = ActivitySource.where(id: params[:id]).by_project(current_project.id).first
    if source 
      if source.update_attributes(ApiSourceAdapter::ParamsCurator.new(source._type.constantize, values).curated_params)
        head :ok
      else
        render json: { errors: source }, status: :unprocessable_entity
      end
    else
      head :not_found
    end
  end
  private :source_updater

end
